# -*- coding: UTF-8 -*-

import ConfigParser
import nes_common.logger as logger
import os
import sys
from artix.common.progress import Progress
from nes_common import sql_operation


def getBoolFromConfig(config, nameOfParam, defaultValue, section):
    return config.getboolean(section, nameOfParam) if config.has_section(section) and config.has_option(section, nameOfParam) else defaultValue


class DataIntegrityError:
    pass


class Executor(object):
    context = None

    def __init__(self, shopCode, connection, context, commandCommitCount):
        self.log = logger.getLogger('Shop', shopCode)
        config = ConfigParser.RawConfigParser()
        if not config.read(os.path.join(os.path.dirname(os.path.realpath(__file__)), '../../config/nes_load.ini')):
            self.log.error("Configuration file 'nes_load.ini' not found")

        self.connection = connection
        self.context = context
        self.trickyCounter = 0
        self.commandCommitCount = commandCommitCount
        self.sqlInsert = None
        self.sqlDelete = None
        self.progress = Progress(self.log, getBoolFromConfig(config, "testProgressMode", False, "MAIN"))

    def startTransaction(self):
        cur = self.connection.cursor()
        cur.execute('START TRANSACTION;')

    def execute(self, sql):
        try:
            sql.execute(self.context)
        except DataIntegrityError:
            self.log.error('data integrity error. exiting')
            sys.exit(1)

    def executeMany(self):
        if self.context.executeMany:
            if self.sqlDelete:
                self.sqlDelete.execMany(self.context)
            if self.sqlInsert:
                self.sqlInsert.execMany(self.context)

    def commit(self):
        if self.trickyCounter >= self.commandCommitCount:
            self.executeMany()
            self.connection.commit()
            self.trickyCounter = 0
        self.trickyCounter += 1

    def rollback(self):
        self.connection.rollback()

    def flushTables(self):
        cur = self.connection.cursor()
        cur.execute('FLUSH tables;')

    def startUpload(self, messageId):
        self.log.info('Начало загрузки справочника')

        try:
            sql_operation._Insert.cache.clear()
            sql_operation._Update.cache.clear()
            sql_operation._Select.cache.clear()
            sql_operation._Delete.cache.clear()
        except Exception as e:
            self.log.error('Error clear cache. %s' % e)

        self.flushTables()
        self.progress.initNesProgress(messageId)

    def finishUpload(self):
        self.log.info(u'Завершение загрузки справочника')
        self.executeMany()
        if not self.context.finalize():
            self.log.info('data inegrity error. exiting')
            sys.exit(1)
        self.connection.commit()
        self.log.info(u"Сброс кэша запросов")
        self.flushTables()
        self.progress.removeProgresses()

    def updateProgress(self):
        """ Обновление прогресса загрузки """
        self.progress.updateNesProgress()
