'''
Created on 13.04.2010

@author: gsu
'''

import mysql.simplesql as sql

tables = ['timeperiod', 'kitdisc', 'kitgoods', 'kit', 'campaign']


def timePeriods(discountAction):
    return discountAction.get('timeperiods', [])


def timePeriodsForDiscountAction(discountAction):
    for tp in timePeriods(discountAction):
        yield sql.Insert(table='timeperiod', values={
            'campaignid': discountAction.get('iddiscountaction', None),
            'date_end': tp.get('dateend', None) if tp.get('dateend', None) else None,
            'date_start': tp.get('datestart', None) if tp.get('datestart', None) else None,
            'day_end': tp.get('dayend', None),
            'day_start': tp.get('daystart', None),
            'time_end': tp.get('timeend', None) if tp.get('timeend', None) else None,
            'time_start': tp.get('timestart', None) if tp.get('timestart', None) else None
        })


def discs(item):
    return item.get('discountrate', {})


def discsForDiscountAction(discountAction):
    for k in kits(discountAction):
        for item in kitInvents(k):
            kitgoodsid = item['kitgoodsid']
            disc = discs(item)
            yield sql.Insert(table='kitdisc', values={
                'disckind': 1 if disc.get('type', None) == 'percent' else 2,
                'disctype': 1 if disc.get('value', 0.0) >= 0.0 else 2,
                'discval': abs(disc.get('value', 0.0)),
                'kitgoodsid': kitgoodsid})


def kitInvents(kit):
    return kit.get('invents', [])


def kitInventsForDiscountAction(discountAction):
    for k in kits(discountAction):
        yield sql.Delete(table='kitgoods', whereFields=['kitid'], whereValues=[k.get('idkit', None)], notExecMany=True)
        for item in kitInvents(k):
            generator = sql.Insert(table='kitgoods', values={
                'goodscode': item.get('inventcode', None),
                'kitid': k.get('idkit', None),
                'quantity': item.get('quantity', None)},
                                   saveLastId=True)
            yield generator
            item['kitgoodsid'] = generator.lastId()


def kits(discountAction):
    return discountAction.get('kits', [])


def kitsForDiscountAction(discountAction):
    for kit in kits(discountAction):
        yield sql.Insert(table='kit', values={
            'campaignid': discountAction.get('iddiscountaction', None),
            'kitid': kit.get('idkit', None),
            'name': kit.get('name', None),
            'kittype': kit.get('kittype', 0),
            'maxtimes': kit.get('maxtimes', None)
        })


def insertDiscountAction(discountAction):
    yield sql.Insert(table='campaign', values={
        'name': discountAction.get('name', ''),
        'campaignid': discountAction.get('iddiscountaction', None)})
    campaignid = discountAction.get('iddiscountaction', None)
    yield sql.Delete(table='timeperiod', whereFields=['campaignid'], whereValues=[campaignid])
    for p in timePeriodsForDiscountAction(discountAction):
        yield p
    kitIds = map(lambda kit: kit.get('idkit', 0), kits(discountAction))
    yield sql.DeleteNotNestedItems(table='kit', mainCodeField='campaignid', mainCodeValue=campaignid, nestedCodeField='kitid', nestedCodeIn=kitIds)
    for k in kitsForDiscountAction(discountAction):
        yield k
    basekits = sql.Select(table='kit', fields=['kitid'], whereFields=['campaignid'], whereValues=[campaignid])
    kitgoods = sql.Select(table='kitgoods', fields=['kitgoodsid'], whereFields=['kitid'], whereValues=basekits)
    yield sql.Delete(table='kitdisc', whereFields=['kitgoodsid'], whereValues=kitgoods, notExecMany=True)
    for ki in kitInventsForDiscountAction(discountAction):
        yield ki
    for d in discsForDiscountAction(discountAction):
        yield d


def deleteDiscountAction(discountAction):
    campaignid = discountAction.get('iddiscountaction', None)
    for k in deleteDiscountActionById(campaignid):
        yield k


def clearDiscountAction():
    yield sql.DeleteAll(table='kit')
    yield sql.DeleteAll(table='kitgoods')
    yield sql.DeleteAll(table='timeperiod')
    yield sql.DeleteAll(table='kitdisc')
    yield sql.DeleteAll(table='campaign')


def deleteDiscountActionById(campaignid):
    kits = sql.Select(table='kit', fields=['kitid'], whereFields=['campaignid'], whereValues=[campaignid])
    kitgoods = sql.Select(table='kitgoods', fields=['kitgoodsid'], whereFields=['kitid'], whereValues=kits)
    yield sql.Delete(table='timeperiod', whereFields=['campaignid'], whereValues=[campaignid])
    yield sql.Delete(table='kitdisc', whereFields=['kitgoodsid'], whereValues=kitgoods)
    yield sql.Delete(table='kitgoods', whereFields=['kitid'], whereValues=kits, notExecMany=True)
    yield sql.Delete(table='kit', whereFields=['campaignid'], whereValues=[campaignid])
    yield sql.Delete(table='campaign', whereFields=['campaignid'], whereValues=[campaignid])
